/**************************************************************/
/*					  Jewel Crafting System 3
/* 
/*	Scripted by Ravenal/awakenlands
/*  Completed on:  5/31/2005
/*
/*	This script is for anyone to use and edit, please be 
/*  considerate not to claim this as your script, thanks.
/*	
/*	Script Site:	   http://jcs3.uniphix.com
/*	Shard's Site:	   http://www.paragonix.org
/*	Developer's Site:  http://www.orbsydia.com
/*	Emulator's Site:   http://www.runuo.com
/*
/*					  Script Version 3.0b
/**************************************************************/

using System;
using Server;
using Server.Items;
using Server.Engines.JewelCraft;

namespace Server.Engines.Craft
{
	public class DefJewelCraft : CraftSystem
	{
		public override SkillName MainSkill
		{
			get	{ return JCore.CraftSkill; }
		}

		public override string GumpTitleString
		{
			get { return String.Format( "<BASEFONT COLOR=WHITE><CENTER>{0} {1}</CENTER></BASEFONT>", JCore.CraftTitle.ToUpper(), JCore.Version ); }
		}

		private static CraftSystem m_CraftSystem;

		public static CraftSystem CraftSystem
		{
			get
			{
				if ( m_CraftSystem == null )
					m_CraftSystem = new DefJewelCraft();

				return m_CraftSystem;
			}
		}

		private DefJewelCraft() : base( 1, 1, 1.25 )
		{
		}
		
		public override double GetChanceAtMin( CraftItem item )
		{
			return 0.0; // 0%
		}

		public override int CanCraft( Mobile from, BaseTool tool, Type itemType )
		{
			if ( tool.Deleted || tool.UsesRemaining < 0 )
				return 1044038; // You have worn out your tool!
			else if ( !BaseTool.CheckAccessible( tool, from ) )
				return 1044263; // The tool must be on your person to use.
			
			return 0;
		}

		public override void PlayCraftEffect( Mobile from )
		{
			JCore.PlayCraftSound( from );
		}

		public override int PlayEndingEffect( Mobile from, bool failed, bool lostMaterial, bool toolBroken, int quality, bool makersMark, CraftItem item )
		{
			if ( toolBroken )
				from.SendLocalizedMessage( 1044038 ); // You have worn out your tool

			if ( failed )
			{
				if ( lostMaterial )
					return 1044043; // You failed to create the item, and some of your materials are lost.
				else
					return 1044157; // You failed to create the item, but no materials were lost.
			}
			else
			{
				if ( makersMark )
				{
					from.SendMessage( "You create the item and affix maker's mark." );
					return 0;					
				}
				else
					return 1044154; // You create the item.
			}
		}
		
		public void AddBracelet( string sectionname, Type itemType, Type barItemType, GemType gemType, Type gemItemType, string jewelryname )
		{
			AddJewelry( sectionname, itemType, barItemType, gemType, gemItemType, jewelryname );
		}
		
		public void AddEarrings( string sectionname, Type itemType, Type barItemType, GemType gemType, Type gemItemType, string jewelryname )
		{
			AddJewelry( sectionname, itemType, barItemType, gemType, gemItemType, jewelryname );
		}
		
		public void AddNecklace( string sectionname, Type itemType, Type barItemType, GemType gemType, Type gemItemType, string jewelryname )
		{
			AddJewelry( sectionname, itemType, barItemType, gemType, gemItemType, jewelryname );
		}
		
		public void AddRing( string sectionname, Type itemType, Type barItemType, GemType gemType, Type gemItemType, string jewelryname )
		{
			AddJewelry( sectionname, itemType, barItemType, gemType, gemItemType, jewelryname );
		}
		
		public void AddJewelry( string sectionname, Type itemType, Type barItemType, GemType gemType, Type gemItemType, string jewelryname )
		{
			string gemname = null;
			string barname = null;	
			
			switch( gemType )
			{
				case GemType.None: gemname = null;	break;
				case GemType.Amber: gemname = "Amber "; break;
				case GemType.Amethyst: gemname = "Amethyst "; break;
				case GemType.Citrine: gemname = "Citrine "; break;
				case GemType.Diamond: gemname = "Diamond "; break;
				case GemType.Emerald: gemname = "Emerald "; break;
				case GemType.Ruby: gemname = "Ruby "; break;
				case GemType.Sapphire: gemname = "Sapphire "; break;
				case GemType.StarSapphire: gemname = "Star Sapphire "; break;
				case GemType.Tourmaline: gemname = "Tourmaline "; break;
				
				//New Gems
				case GemType.FireEmerald: gemname = "Fire Emerald "; break;
				case GemType.Onyx: gemname = "Onyx "; break;
				case GemType.StarRuby: gemname = "Star Ruby "; break;
				case GemType.BlueDiamond: gemname = "Blue Diamond "; break;
				case GemType.FireOpal: gemname = "Fire Opal "; break;
				case GemType.Jade: gemname = "Jade "; break;
				case GemType.Jasper: gemname = "Jasper "; break;
				case GemType.Pearl: gemname = "Pearl "; break;
				case GemType.Opal: gemname = "Opal "; break;
				case GemType.StarRose: gemname = "Star Rose "; break;
				case GemType.Topaz: gemname = "Topaz "; break;
				case GemType.Turquoise: gemname = "Turquoise "; break;
			}

			switch( barItemType.Name )
			{
				case "CopperBar": barname = "Copper Bar"; break;
				case "SilverBar": barname = "Silver Bar"; break;
				case "GoldBar": barname = "Gold Bar"; break;
				case "PlatinumBar": barname = "Platinum Bar"; break;
				case "VeliumBar": barname = "Velium Bar"; break;
				case "ArycilanBar": barname = "Arycilan Bar"; break;
				case "OrbsyidanBar": barname = "Orbsydian Bar"; break;
				case "XulianBar": barname = "Xulian Bar"; break;
				case "ParagonBar": barname = "Paragon Bar"; break;
			}

			double min = JCore.GetGemMinSkill( gemType );
			double max = JCore.GetGemMaxSkill( gemType );
							
			int index = AddCraft( itemType, sectionname, gemname+jewelryname, min, max, barItemType, barname, 1, 1044037 );
			
			if( gemItemType != null )
				AddRes( index, gemItemType, gemname, 1, 1044253 );
		}

		public override void InitCraftList()
		{
			#region Rings
			AddRing( "Rings", typeof( Ring ), typeof( CopperBar ), GemType.None, null, "Ring" );
			
			AddRing( "Rings", typeof( Ring ), typeof( CopperBar ), GemType.Amber, typeof( Amber ), "Ring" );
			AddRing( "Rings", typeof( Ring ), typeof( CopperBar ), GemType.Amethyst, typeof( Amethyst ), "Ring" );
			AddRing( "Rings", typeof( Ring ), typeof( CopperBar ), GemType.Citrine, typeof( Citrine ), "Ring" );
			AddRing( "Rings", typeof( Ring ), typeof( CopperBar ), GemType.Diamond, typeof( Diamond ), "Ring" );
			AddRing( "Rings", typeof( Ring ), typeof( CopperBar ), GemType.Emerald, typeof( Emerald ), "Ring" );
			AddRing( "Rings", typeof( Ring ), typeof( CopperBar ), GemType.Ruby, typeof( Ruby ), "Ring" );
			AddRing( "Rings", typeof( Ring ), typeof( CopperBar ), GemType.Sapphire, typeof( Sapphire ), "Ring" );
			AddRing( "Rings", typeof( Ring ), typeof( CopperBar ), GemType.StarSapphire, typeof( StarSapphire ), "Ring" );
			AddRing( "Rings", typeof( Ring ), typeof( CopperBar ), GemType.Tourmaline, typeof( Tourmaline ), "Ring" );
			AddRing( "Rings", typeof( Ring ), typeof( CopperBar ), GemType.FireEmerald, typeof( FireEmerald ), "Ring" );
			AddRing( "Rings", typeof( Ring ), typeof( CopperBar ), GemType.Onyx, typeof( Onyx ), "Ring" );
			AddRing( "Rings", typeof( Ring ), typeof( CopperBar ), GemType.StarRuby, typeof( StarRuby ), "Ring" );
			AddRing( "Rings", typeof( Ring ), typeof( CopperBar ), GemType.BlueDiamond, typeof( BlueDiamond ), "Ring" );
			AddRing( "Rings", typeof( Ring ), typeof( CopperBar ), GemType.FireOpal, typeof( FireOpal ), "Ring" );
			AddRing( "Rings", typeof( Ring ), typeof( CopperBar ), GemType.Jade, typeof( Jade ), "Ring" );
			AddRing( "Rings", typeof( Ring ), typeof( CopperBar ), GemType.Jasper, typeof( Jasper ), "Ring" );
			AddRing( "Rings", typeof( Ring ), typeof( CopperBar ), GemType.Pearl, typeof( Pearl ), "Ring" );
			AddRing( "Rings", typeof( Ring ), typeof( CopperBar ), GemType.Opal, typeof( Opal ), "Ring" );
			AddRing( "Rings", typeof( Ring ), typeof( CopperBar ), GemType.StarRose, typeof( StarRose ), "Ring" );
			AddRing( "Rings", typeof( Ring ), typeof( CopperBar ), GemType.Topaz, typeof( Topaz ), "Ring" );
			AddRing( "Rings", typeof( Ring ), typeof( CopperBar ), GemType.Turquoise, typeof( Turquoise ), "Ring" );
			#endregion
			
			#region Braclets
			AddBracelet( "Bracelets", typeof( Bracelet ), typeof( CopperBar ), GemType.None, null, "Bracelet" );
			
			AddBracelet( "Bracelets", typeof( Bracelet ), typeof( CopperBar ), GemType.Amber, typeof( Amber ), "Bracelet" );
			AddBracelet( "Bracelets", typeof( Bracelet ), typeof( CopperBar ), GemType.Amethyst, typeof( Amethyst ), "Bracelet" );
			AddBracelet( "Bracelets", typeof( Bracelet ), typeof( CopperBar ), GemType.Citrine, typeof( Citrine ), "Bracelet" );
			AddBracelet( "Bracelets", typeof( Bracelet ), typeof( CopperBar ), GemType.Diamond, typeof( Diamond ), "Bracelet" );
			AddBracelet( "Bracelets", typeof( Bracelet ), typeof( CopperBar ), GemType.Emerald, typeof( Emerald ), "Bracelet" );
			AddBracelet( "Bracelets", typeof( Bracelet ), typeof( CopperBar ), GemType.Ruby, typeof( Ruby ), "Bracelet" );
			AddBracelet( "Bracelets", typeof( Bracelet ), typeof( CopperBar ), GemType.Sapphire, typeof( Sapphire ), "Bracelet" );
			AddBracelet( "Bracelets", typeof( Bracelet ), typeof( CopperBar ), GemType.StarSapphire, typeof( StarSapphire ), "Bracelet" );
			AddBracelet( "Bracelets", typeof( Bracelet ), typeof( CopperBar ), GemType.Tourmaline, typeof( Tourmaline ), "Bracelet" );
			AddBracelet( "Bracelets", typeof( Bracelet ), typeof( CopperBar ), GemType.FireEmerald, typeof( FireEmerald ), "Bracelet" );
			AddBracelet( "Bracelets", typeof( Bracelet ), typeof( CopperBar ), GemType.Onyx, typeof( Onyx ), "Bracelet" );
			AddBracelet( "Bracelets", typeof( Bracelet ), typeof( CopperBar ), GemType.StarRuby, typeof( StarRuby ), "Bracelet" );
			AddBracelet( "Bracelets", typeof( Bracelet ), typeof( CopperBar ), GemType.BlueDiamond, typeof( BlueDiamond ), "Bracelet" );
			AddBracelet( "Bracelets", typeof( Bracelet ), typeof( CopperBar ), GemType.FireOpal, typeof( FireOpal ), "Bracelet" );
			AddBracelet( "Bracelets", typeof( Bracelet ), typeof( CopperBar ), GemType.Jade, typeof( Jade ), "Bracelet" );
			AddBracelet( "Bracelets", typeof( Bracelet ), typeof( CopperBar ), GemType.Jasper, typeof( Jasper ), "Bracelet" );
			AddBracelet( "Bracelets", typeof( Bracelet ), typeof( CopperBar ), GemType.Pearl, typeof( Pearl ), "Bracelet" );
			AddBracelet( "Bracelets", typeof( Bracelet ), typeof( CopperBar ), GemType.Opal, typeof( Opal ), "Bracelet" );
			AddBracelet( "Bracelets", typeof( Bracelet ), typeof( CopperBar ), GemType.StarRose, typeof( StarRose ), "Bracelet" );
			AddBracelet( "Bracelets", typeof( Bracelet ), typeof( CopperBar ), GemType.Topaz, typeof( Topaz ), "Bracelet" );
			AddBracelet( "Bracelets", typeof( Bracelet ), typeof( CopperBar ), GemType.Turquoise, typeof( Turquoise ), "Bracelet" );
			#endregion
			
			#region Necklaces
			AddNecklace( "Necklaces - Normal", typeof( Necklace ), typeof( CopperBar ), GemType.None, null, "Necklace" );
			
			AddNecklace( "Necklaces - Normal", typeof( Necklace ), typeof( CopperBar ), GemType.Amber, typeof( Amber ), "Necklace" );
			AddNecklace( "Necklaces - Normal", typeof( Necklace ), typeof( CopperBar ), GemType.Amethyst, typeof( Amethyst ), "Necklace" );
			AddNecklace( "Necklaces - Normal", typeof( Necklace ), typeof( CopperBar ), GemType.Citrine, typeof( Citrine ), "Necklace" );
			AddNecklace( "Necklaces - Normal", typeof( Necklace ), typeof( CopperBar ), GemType.Diamond, typeof( Diamond ), "Necklace" );
			AddNecklace( "Necklaces - Normal", typeof( Necklace ), typeof( CopperBar ), GemType.Emerald, typeof( Emerald ), "Necklace" );
			AddNecklace( "Necklaces - Normal", typeof( Necklace ), typeof( CopperBar ), GemType.Ruby, typeof( Ruby ), "Necklace" );
			AddNecklace( "Necklaces - Normal", typeof( Necklace ), typeof( CopperBar ), GemType.Sapphire, typeof( Sapphire ), "Necklace" );
			AddNecklace( "Necklaces - Normal", typeof( Necklace ), typeof( CopperBar ), GemType.StarSapphire, typeof( StarSapphire ), "Necklace" );
			AddNecklace( "Necklaces - Normal", typeof( Necklace ), typeof( CopperBar ), GemType.Tourmaline, typeof( Tourmaline ), "Necklace" );
			AddNecklace( "Necklaces - Normal", typeof( Necklace ), typeof( CopperBar ), GemType.FireEmerald, typeof( FireEmerald ), "Necklace" );
			AddNecklace( "Necklaces - Normal", typeof( Necklace ), typeof( CopperBar ), GemType.Onyx, typeof( Onyx ), "Necklace" );
			AddNecklace( "Necklaces - Normal", typeof( Necklace ), typeof( CopperBar ), GemType.StarRuby, typeof( StarRuby ), "Necklace" );
			AddNecklace( "Necklaces - Normal", typeof( Necklace ), typeof( CopperBar ), GemType.BlueDiamond, typeof( BlueDiamond ), "Necklace" );
			AddNecklace( "Necklaces - Normal", typeof( Necklace ), typeof( CopperBar ), GemType.FireOpal, typeof( FireOpal ), "Necklace" );
			AddNecklace( "Necklaces - Normal", typeof( Necklace ), typeof( CopperBar ), GemType.Jade, typeof( Jade ), "Necklace" );
			AddNecklace( "Necklaces - Normal", typeof( Necklace ), typeof( CopperBar ), GemType.Jasper, typeof( Jasper ), "Necklace" );
			AddNecklace( "Necklaces - Normal", typeof( Necklace ), typeof( CopperBar ), GemType.Pearl, typeof( Pearl ), "Necklace" );
			AddNecklace( "Necklaces - Normal", typeof( Necklace ), typeof( CopperBar ), GemType.Opal, typeof( Opal ), "Necklace" );
			AddNecklace( "Necklaces - Normal", typeof( Necklace ), typeof( CopperBar ), GemType.StarRose, typeof( StarRose ), "Necklace" );
			AddNecklace( "Necklaces - Normal", typeof( Necklace ), typeof( CopperBar ), GemType.Topaz, typeof( Topaz ), "Necklace" );
			AddNecklace( "Necklaces - Normal", typeof( Necklace ), typeof( CopperBar ), GemType.Turquoise, typeof( Turquoise ), "Necklace" );

			AddNecklace( "Necklaces - Bead", typeof( BeadNecklace ), typeof( CopperBar ), GemType.None, null, "Bead Necklace" );
			
			AddNecklace( "Necklaces - Bead", typeof( BeadNecklace ), typeof( CopperBar ), GemType.Amber, typeof( Amber ), "Bead Necklace" );
			AddNecklace( "Necklaces - Bead", typeof( BeadNecklace ), typeof( CopperBar ), GemType.Amethyst, typeof( Amethyst ), "Bead Necklace" );
			AddNecklace( "Necklaces - Bead", typeof( BeadNecklace ), typeof( CopperBar ), GemType.Citrine, typeof( Citrine ), "Bead Necklace" );
			AddNecklace( "Necklaces - Bead", typeof( BeadNecklace ), typeof( CopperBar ), GemType.Diamond, typeof( Diamond ), "Bead Necklace" );
			AddNecklace( "Necklaces - Bead", typeof( BeadNecklace ), typeof( CopperBar ), GemType.Emerald, typeof( Emerald ), "Bead Necklace" );
			AddNecklace( "Necklaces - Bead", typeof( BeadNecklace ), typeof( CopperBar ), GemType.Ruby, typeof( Ruby ), "Bead Necklace" );
			AddNecklace( "Necklaces - Bead", typeof( BeadNecklace ), typeof( CopperBar ), GemType.Sapphire, typeof( Sapphire ), "Bead Necklace" );
			AddNecklace( "Necklaces - Bead", typeof( BeadNecklace ), typeof( CopperBar ), GemType.StarSapphire, typeof( StarSapphire ), "Bead Necklace" );
			AddNecklace( "Necklaces - Bead", typeof( BeadNecklace ), typeof( CopperBar ), GemType.Tourmaline, typeof( Tourmaline ), "Bead Necklace" );
			AddNecklace( "Necklaces - Bead", typeof( BeadNecklace ), typeof( CopperBar ), GemType.FireEmerald, typeof( FireEmerald ), "Bead Necklace" );
			AddNecklace( "Necklaces - Bead", typeof( BeadNecklace ), typeof( CopperBar ), GemType.Onyx, typeof( Onyx ), "Bead Necklace" );
			AddNecklace( "Necklaces - Bead", typeof( BeadNecklace ), typeof( CopperBar ), GemType.StarRuby, typeof( StarRuby ), "Bead Necklace" );
			AddNecklace( "Necklaces - Bead", typeof( BeadNecklace ), typeof( CopperBar ), GemType.BlueDiamond, typeof( BlueDiamond ), "Bead Necklace" );
			AddNecklace( "Necklaces - Bead", typeof( BeadNecklace ), typeof( CopperBar ), GemType.FireOpal, typeof( FireOpal ), "Bead Necklace" );
			AddNecklace( "Necklaces - Bead", typeof( BeadNecklace ), typeof( CopperBar ), GemType.Jade, typeof( Jade ), "Bead Necklace" );
			AddNecklace( "Necklaces - Bead", typeof( BeadNecklace ), typeof( CopperBar ), GemType.Jasper, typeof( Jasper ), "Bead Necklace" );
			AddNecklace( "Necklaces - Bead", typeof( BeadNecklace ), typeof( CopperBar ), GemType.Pearl, typeof( Pearl ), "Bead Necklace" );
			AddNecklace( "Necklaces - Bead", typeof( BeadNecklace ), typeof( CopperBar ), GemType.Opal, typeof( Opal ), "Bead Necklace" );
			AddNecklace( "Necklaces - Bead", typeof( BeadNecklace ), typeof( CopperBar ), GemType.StarRose, typeof( StarRose ), "Bead Necklace" );
			AddNecklace( "Necklaces - Bead", typeof( BeadNecklace ), typeof( CopperBar ), GemType.Topaz, typeof( Topaz ), "Bead Necklace" );
			AddNecklace( "Necklaces - Bead", typeof( BeadNecklace ), typeof( CopperBar ), GemType.Turquoise, typeof( Turquoise ), "Bead Necklace" );
			#endregion
			
			#region Earrings
			AddEarrings( "Earrings", typeof( Earrings ), typeof( CopperBar ), GemType.None, null, "Earring" );
			
			AddEarrings( "Earrings", typeof( Earrings ), typeof( CopperBar ), GemType.Amber, typeof( Amber ), "Earring" );
			AddEarrings( "Earrings", typeof( Earrings ), typeof( CopperBar ), GemType.Amethyst, typeof( Amethyst ), "Earring" );
			AddEarrings( "Earrings", typeof( Earrings ), typeof( CopperBar ), GemType.Citrine, typeof( Citrine ), "Earring" );
			AddEarrings( "Earrings", typeof( Earrings ), typeof( CopperBar ), GemType.Diamond, typeof( Diamond ), "Earring" );
			AddEarrings( "Earrings", typeof( Earrings ), typeof( CopperBar ), GemType.Emerald, typeof( Emerald ), "Earring" );
			AddEarrings( "Earrings", typeof( Earrings ), typeof( CopperBar ), GemType.Ruby, typeof( Ruby ), "Earring" );
			AddEarrings( "Earrings", typeof( Earrings ), typeof( CopperBar ), GemType.Sapphire, typeof( Sapphire ), "Earring" );
			AddEarrings( "Earrings", typeof( Earrings ), typeof( CopperBar ), GemType.StarSapphire, typeof( StarSapphire ), "Earring" );
			AddEarrings( "Earrings", typeof( Earrings ), typeof( CopperBar ), GemType.Tourmaline, typeof( Tourmaline ), "Earring" );
			AddEarrings( "Earrings", typeof( Earrings ), typeof( CopperBar ), GemType.FireEmerald, typeof( FireEmerald ), "Earring" );
			AddEarrings( "Earrings", typeof( Earrings ), typeof( CopperBar ), GemType.Onyx, typeof( Onyx ), "Earring" );
			AddEarrings( "Earrings", typeof( Earrings ), typeof( CopperBar ), GemType.StarRuby, typeof( StarRuby ), "Earring" );
			AddEarrings( "Earrings", typeof( Earrings ), typeof( CopperBar ), GemType.BlueDiamond, typeof( BlueDiamond ), "Earring" );
			AddEarrings( "Earrings", typeof( Earrings ), typeof( CopperBar ), GemType.FireOpal, typeof( FireOpal ), "Earring" );
			AddEarrings( "Earrings", typeof( Earrings ), typeof( CopperBar ), GemType.Jade, typeof( Jade ), "Earring" );
			AddEarrings( "Earrings", typeof( Earrings ), typeof( CopperBar ), GemType.Jasper, typeof( Jasper ), "Earring" );
			AddEarrings( "Earrings", typeof( Earrings ), typeof( CopperBar ), GemType.Pearl, typeof( Pearl ), "Earring" );
			AddEarrings( "Earrings", typeof( Earrings ), typeof( CopperBar ), GemType.Opal, typeof( Opal ), "Earring" );
			AddEarrings( "Earrings", typeof( Earrings ), typeof( CopperBar ), GemType.StarRose, typeof( StarRose ), "Earring" );
			AddEarrings( "Earrings", typeof( Earrings ), typeof( CopperBar ), GemType.Topaz, typeof( Topaz ), "Earring" );
			AddEarrings( "Earrings", typeof( Earrings ), typeof( CopperBar ), GemType.Turquoise, typeof( Turquoise ), "Earring" );
			#endregion

			SetSubRes( typeof( CopperBar ), "		COPPER BAR" );

			// Add every material you want the player to be able to chose from
			// This will override the overridable material
			AddSubRes( typeof( CopperBar ),			"COPPER BAR",		00.0,	1044268 );
			AddSubRes( typeof( SilverBar ),			"SILVER BAR",		65.0,	1044268 );
			AddSubRes( typeof( GoldBar ),			"GOLD BAR",			70.0,	1044268 );
			AddSubRes( typeof( PlatinumBar ),		"PLATINUM BAR",		75.0,	1044268 );
			AddSubRes( typeof( VeliumBar ),			"VELIUM BAR",		80.0,	1044268 );
			AddSubRes( typeof( ArycilanBar ),		"ARYCILAN BAR",		85.0,	1044268 );
			AddSubRes( typeof( OrbsydianBar ),		"ORBSYDIAN BAR",	90.0,	1044268 );
			AddSubRes( typeof( XulianBar ),			"XULIAN BAR",		95.0,	1044268 );
			AddSubRes( typeof( ParagonBar ),		"PARAGON BAR",		99.0,	1044268 );
			
			MarkOption = true;
		}
	}
}
