/**************************************************************/
/*					  Jewel Crafting System 3
/* 
/*	Scripted by Ravenal/awakenlands
/*  Completed on:  5/31/2005
/*
/*	This script is for anyone to use and edit, please be 
/*  considerate not to claim this as your script, thanks.
/*	
/*	Script Site:	   http://jcs3.uniphix.com
/*	Shard's Site:	   http://www.paragonix.org
/*	Developer's Site:  http://www.orbsydia.com
/*	Emulator's Site:   http://www.runuo.com
/*
/*					  Script Version 3.0
/**************************************************************/

using System;
using Server;
using Server.Targeting;
using Server.Targets;
using Server.Engines.JewelCraft;

namespace Server.Items
{
	public class EnchantedScroll : Item, Server.Items.ICommodity
	{
		#region ICommodity Members

		public string Description
		{
			get
			{
				return String.Format( Amount == 1 ? "{0} enchanted scroll" : "{0} enchanted scrolls", Amount );
			}
		}

		#endregion

		[Constructable]
		public EnchantedScroll() : this( 1 )
		{
		}

		[Constructable]
		public EnchantedScroll( int amount ) : base( 0x1F6D )
		{
			Name = "enchanted scroll";
			Hue = JCore.EnchantedScrollHue;
			Stackable = true;
			Weight = 1.0;
			Amount = amount;
		}

		public EnchantedScroll( Serial serial ) : base( serial )
		{
		}

		public override void OnDoubleClick(Mobile from)
		{
			from.SendMessage( "Target a jewelry of your choice to enchant." );
		
			from.Target = new EnchantTarget(this);
		}   

		private bool CheckMana( Mobile m )
		{
			if( m.Mana <= JCore.ManaReq )
				return true;

			return false;
		}

		public virtual void OnTarget( Mobile m, BaseJewel j )
		{
			if( JCore.Enabled )
			{
				if( CheckMana( m ) )
				{
					if( j.IsEnchantable )
					{
						// This checks your skill to see if your are GM and checks your Tinkering Cap for the MaxSkill check
						// It also checks to see if you have an enchant scroll inside your pack, if so it will consume one of them.			
						if( !m.CheckSkill( SkillName.Tinkering, 100.0, m.Skills[SkillName.Tinkering].Cap ) )
						{
							m.SendMessage( "You attempt to enchant the item but fail. You loose the jewelry since it exploded as you attempt to enchant." ); 
							m.PlaySound( 519 );
							m.FixedParticles( 0x36BD, 20, 10, 5044, EffectLayer.Head );
							m.Damage( Utility.RandomMinMax( 10, 20 ) );
							m.Mana -= JCore.ManaReq;
							this.Consume(1);
							j.Delete();
						}
						else
						{   					
							// At GM Tinkering you have a 25% chance of enchanting the item.
							if( (m.Skills[SkillName.Tinkering].Value * 0.0025 ) >= Utility.RandomDouble() )
							{
								m.SendMessage( "You sucessfully enchant this item!" );
								j.Enchanted = true;
								Effects.SendLocationEffect( new Point3D( m.X, m.Y, m.Z+5 ), m.Map, 14201, 16 );
								m.PlaySound( 491 );
								m.Mana -= JCore.ManaReq;
								this.Consume(1);
							}
							else
							{
								m.SendMessage( "You attempt to enchant the item but fail. You loose the jewelry since it exploded as you attempt to enchant." ); 
								m.PlaySound( 519 );
								m.FixedParticles( 0x36BD, 20, 10, 5044, EffectLayer.Head );
								m.Damage( Utility.RandomMinMax( 10, 20 ) );
								this.Consume(1);
								m.Mana -= JCore.ManaReq;
								j.Delete();
							}
						}
					}
					else
					{
						if( j.Enchanted )
							m.SendMessage( "This jewelry has already been enchanted, you cannot enchant it again." );
						else if( j.Enchanted && j.Imbued )
							m.SendMessage( "This jewelry is both enchanted and imbued already you cannot enchant it." );
						else
							m.SendMessage( "This jewelry is already magical, you cannot enchant it." );
					}
				}
				else
				{
					m.SendMessage( "You do not have enough mana to enchant this jewelry." ); 
				}
			}	
			else
			{
				m.SendMessage( "Jewel Craft System {0} is not enabled!", JCore.Version );
			}
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}

		private class EnchantTarget	: Target
		{
			private EnchantedScroll m_Scroll;

			public EnchantTarget( EnchantedScroll scroll ) :base( -1, false, TargetFlags.None )
			{
				m_Scroll = scroll;
			}

			protected override void OnTarget( Mobile m, object o )
			{
				if( o is BaseJewel && o != null )
				{
					m_Scroll.OnTarget( m, ((BaseJewel)o) );
				}
				else
				{
					m.SendMessage( "That is not a jewelry." );
				}				
			}
		}
	}
}