/**************************************************************/
/*					  Jewel Crafting System 3
/* 
/*	Scripted by Ravenal/awakenlands
/*  Completed on:  5/31/2005
/*
/*	This script is for anyone to use and edit, please be 
/*  considerate not to claim this as your script, thanks.
/*	
/*	Script Site:	   http://jcs3.uniphix.com
/*	Shard's Site:	   http://www.paragonix.org
/*	Developer's Site:  http://www.orbsydia.com
/*	Emulator's Site:   http://www.runuo.com
/*
/*					  Script Version 3.0
/**************************************************************/

using System;
using Server.Engines.JewelCraft;

namespace Server.Items
{
	public abstract class BaseBars : Item
	{
		private BarType m_BarType;
		private string m_OldName;
		
		[CommandProperty( AccessLevel.GameMaster )]
		public BarType BarType
		{
			get{ return m_BarType; }
			set
			{ 
				m_BarType = value; 
				
				JCore.SetHueAndItemID(this);
				
				InvalidateProperties(); 
			}
		}
		
		[CommandProperty( AccessLevel.Administrator )]
		public string OldName
		{
			get{ return m_OldName; }
			set{ m_OldName = value; InvalidateProperties();  }
		}
		
		public BaseBars( int amount, int itemid, BarType type ):base( itemid )
		{
			m_BarType = type;
			Stackable = true;
			Amount = amount;
			this.SetName( "bar" );
			JCore.SetHueAndItemID(this);
		}
		
		public BaseBars( Serial s ):base( s )
		{
		}
		
		public void SetName( string name )
		{
			m_OldName = name;
			this.Name = name;
			
			JCore.GetNames(this);
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version

			// Version 0
			writer.WriteEncodedInt( (int)m_BarType );
			writer.Write( m_OldName );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();

			switch ( version )
			{
				case 0:
				{
					m_BarType = (BarType)reader.ReadEncodedInt();
					m_OldName = reader.ReadString();
					break;
				}
			}
		}
	}
	
	public class ParagonBar : BaseBars
	{
		[Constructable]
		public ParagonBar() : this( 0 )
		{
		}
	
		[Constructable]
		public ParagonBar( int amount ) : base( amount, 0x1bf5, BarType.Paragon )
		{
			Weight = 0.1;
		}

		public ParagonBar( Serial serial ) : base( serial )
		{
		}
		
		public override Item Dupe( int amount )
		{
			return base.Dupe( new ParagonBar( amount ), amount );
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}
	
	public class XulianBar : BaseBars
	{
		[Constructable]
		public XulianBar() : this( 0 )
		{
		}
	
		[Constructable]
		public XulianBar( int amount ) : base( amount, 0x1bf5, BarType.Xulian )
		{
			Weight = 0.1;
		}

		public XulianBar( Serial serial ) : base( serial )
		{
		}
		
		public override Item Dupe( int amount )
		{
			return base.Dupe( new XulianBar( amount ), amount );
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}
	
	public class OrbsydianBar : BaseBars
	{
		[Constructable]
		public OrbsydianBar() : this( 0 )
		{
		}
	
		[Constructable]
		public OrbsydianBar( int amount ) : base( amount, 0x1bf5, BarType.Orbsydian )
		{
			Weight = 0.1;
		}

		public OrbsydianBar( Serial serial ) : base( serial )
		{
		}
		
		public override Item Dupe( int amount )
		{
			return base.Dupe( new OrbsydianBar( amount ), amount );
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}
	
	public class ArycilanBar : BaseBars
	{
		[Constructable]
		public ArycilanBar() : this( 0 )
		{
		}
	
		[Constructable]
		public ArycilanBar( int amount ) : base( amount, 0x1bf5, BarType.Arycilan )
		{
			Weight = 0.1;
		}

		public ArycilanBar( Serial serial ) : base( serial )
		{
		}
		
		public override Item Dupe( int amount )
		{
			return base.Dupe( new ArycilanBar( amount ), amount );
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}
		
	public class VeliumBar : BaseBars
	{
		[Constructable]
		public VeliumBar() : this( 0 )
		{
		}
	
		[Constructable]
		public VeliumBar( int amount ) : base( amount, 0x1bf5, BarType.Velium )
		{
			Weight = 0.1;
		}

		public VeliumBar( Serial serial ) : base( serial )
		{
		}
		
		public override Item Dupe( int amount )
		{
			return base.Dupe( new VeliumBar( amount ), amount );
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}
	
	public class PlatinumBar : BaseBars
	{
		[Constructable]
		public PlatinumBar() :this( 0 )
		{
		}
	
		[Constructable]
		public PlatinumBar( int amount ) : base( amount, 0x1bf5, BarType.Platinum )
		{
			Weight = 0.1;
		}

		public PlatinumBar( Serial serial ) : base( serial )
		{
		}
		
		public override Item Dupe( int amount )
		{
			return base.Dupe( new PlatinumBar( amount ), amount );
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}
	
	public class GoldBar : BaseBars
	{
		[Constructable]
		public GoldBar() :this( 0 )
		{
		}
		
		[Constructable]
		public GoldBar( int amount ) : base( amount, 0x1be9, BarType.Gold )
		{
			Weight = 0.1;
		}

		public GoldBar( Serial serial ) : base( serial )
		{
		}
		
		public override Item Dupe( int amount )
		{
			return base.Dupe( new GoldBar( amount ), amount );
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}
	
	public class SilverBar : BaseBars
	{
		[Constructable]
		public SilverBar() :this( 0 )
		{
		}
		
		[Constructable]
		public SilverBar( int amount ) : base( amount, 0x1bf5, BarType.Silver )
		{
			Weight = 0.1;
		}

		public SilverBar( Serial serial ) : base( serial )
		{
		}
		
		public override Item Dupe( int amount )
		{
			return base.Dupe( new SilverBar( amount ), amount );
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}
	
	public class CopperBar : BaseBars
	{
		[Constructable]
		public CopperBar() :this( 0 )
		{
		}
		
		[Constructable]
		public CopperBar( int amount ) : base( amount, 0x1be3, BarType.Copper )
		{
			Weight = 0.1;
		}

		public CopperBar( Serial serial ) : base( serial )
		{
		}
		
		public override Item Dupe( int amount )
		{
			return base.Dupe( new CopperBar( amount ), amount );
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			writer.Write( (int) 0 ); // version
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();
		}
	}
}
