/**************************************************************/
/*					  Jewel Crafting System 3
/* 
/*	Scripted by Ravenal/awakenlands
/*  Completed on:  5/31/2005
/*
/*	This script is for anyone to use and edit, please be 
/*  considerate not to claim this as your script, thanks.
/*
/*	This Script was heavily edited... 
/*	
/*	Script Site:	   http://jcs3.uniphix.com
/*	Shard's Site:	   http://www.paragonix.org
/*	Developer's Site:  http://www.orbsydia.com
/*	Emulator's Site:   http://www.runuo.com
/*
/*					  Script Version 3.0d
/**************************************************************/

using System;
using Server;
using Server.Engines.Craft;
using Server.Engines.JewelCraft;

namespace Server.Items
{			
	public enum GemType
	{
		None,
		StarSapphire,
		Emerald,
		Sapphire,
		Ruby,
		Citrine,
		Amethyst,
		Tourmaline,
		Amber,
		Diamond,
		#region JewelCrafting System
		FireEmerald,
		Onyx,
		StarRuby,
		Jade,
		Jasper,
		FireOpal,
		Pearl,
		Opal,
		Topaz,
		Turquoise,
		StarRose,
		BlueDiamond
		#endregion
	}

	public abstract class BaseJewel : Item, ICraftable
	{
		private AosAttributes m_AosAttributes;
		private AosElementAttributes m_AosResistances;
		private AosSkillBonuses m_AosSkillBonuses;
		private CraftResource m_Resource;
		private GemType m_GemType;
		
		#region JewelCrafting System
		private Mobile m_Crafter;
		private BarType m_BarType;
		private bool m_Enchanted;
		private bool m_Imbued;
		private string m_OldName;
		private int m_ArtifactRarity;
		
		[CommandProperty( AccessLevel.GameMaster )]
		public Mobile Crafter
		{
			get{ return m_Crafter; }
			set{ m_Crafter = value; InvalidateProperties(); }
		}
		
		[CommandProperty( AccessLevel.GameMaster )]
		public BarType BarType
		{
			get{ return m_BarType; }
			set
			{ 
				this.RemoveMagicalProperties();
				m_BarType = value; 
				
				JCore.SetHue(this);
				JCore.GetNames(this);
				
				InvalidateProperties();
				this.InvalidateMagicProperties(); 
			}
		}
		
		[CommandProperty( AccessLevel.GameMaster )]
		public bool Enchanted
		{
			get{ return m_Enchanted; }
			set
			{ 
				this.RemoveMagicalProperties();
				m_Enchanted = value;
				
				this.InvalidateMagicProperties(); 
				InvalidateProperties(); 
			}
		}
		
		[CommandProperty( AccessLevel.GameMaster )]
		public bool Imbued
		{
			get{ return m_Imbued; }
			set
			{ 
				this.RemoveMagicalProperties();
				m_Imbued = value;
				
				this.InvalidateMagicProperties(); 
				InvalidateProperties(); 
			}
		}
		
		[CommandProperty( AccessLevel.Administrator )]
		public string OldName
		{
			get
			{ 
				return m_OldName;
			}
			set
			{
				m_OldName = value;
			}
		}
		
		#endregion

		[CommandProperty( AccessLevel.GameMaster )]
		public AosAttributes Attributes
		{
			get{ return m_AosAttributes; }
			set{}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public AosElementAttributes Resistances
		{
			get{ return m_AosResistances; }
			set{}
		}

		[CommandProperty( AccessLevel.GameMaster )]
		public AosSkillBonuses SkillBonuses
		{
			get{ return m_AosSkillBonuses; }
			set{}
		}
		
		#region Edited for Jewelcraft
		[CommandProperty( AccessLevel.GameMaster )]
		public CraftResource Resource
		{
			get{ return m_Resource; }
			set
			{ 
				m_Resource = value; 
				
				if( m_GemType == GemType.None || m_BarType == BarType.None )
					Hue = CraftResources.GetHue( m_Resource );
			}
		}		
		
		[CommandProperty( AccessLevel.GameMaster )]
		public GemType GemType
		{
			get{ return m_GemType; }
			set
			{ 
				this.RemoveMagicalProperties();
				m_GemType = value; 
				
				JCore.SetHue(this);
				JCore.GetNames(this);
				
				this.InvalidateMagicProperties();
				InvalidateProperties(); 
			}
		}
		#endregion

		public override int PhysicalResistance{ get{ return m_AosResistances.Physical; } }
		public override int FireResistance{ get{ return m_AosResistances.Fire; } }
		public override int ColdResistance{ get{ return m_AosResistances.Cold; } }
		public override int PoisonResistance{ get{ return m_AosResistances.Poison; } }
		public override int EnergyResistance{ get{ return m_AosResistances.Energy; } }
		public virtual int BaseGemTypeNumber{ get{ return 0; } }

		#region Edited for Jewel Craft
		/// <summary>
		/// If JCore (Jewel Crafting) Is Enabled, the Label Number is Based off of Default!
		/// Jewelry Crafting uses the new Names that are fired with the MagicalProperties are Fired
		/// </summary>
		public override int LabelNumber
		{
			get
			{
				if ( JCore.Enabled )
					return base.LabelNumber;
				
				if ( m_GemType == GemType.None )
					return base.LabelNumber;

				return BaseGemTypeNumber + (int)m_GemType - 1;
			}
		}
		#endregion

		#region Edited for Jewel Crafting

		/// <summary>
		/// ArtifactRarity is NOW stored...
		/// This is not Overridable no longer...
		/// </summary>
		[CommandProperty( AccessLevel.GameMaster )]
		public int ArtifactRarity
		{ 
			get{ return m_ArtifactRarity; } 
			set{ m_ArtifactRarity = value; }	
		}

		public BaseJewel( int itemID, BarType type, Layer layer ) : base( itemID )
		{
			m_BarType = type;
			m_AosAttributes = new AosAttributes( this );
			m_AosResistances = new AosElementAttributes( this );
			m_AosSkillBonuses = new AosSkillBonuses( this );
			m_Resource = CraftResource.None;
			m_GemType = GemType.None;
			
			m_Enchanted = false;
			m_Imbued = false;
			m_Crafter = null;
			JCore.SetHue(this);
			SetName( this.Name );
			Layer = layer;					
		}
		
		public void SetName( string name )
		{
			m_OldName = name;
			this.Name = name;
			
			JCore.GetNames(this);
		}
		#endregion

		public override void OnAdded( object parent )
		{
			if ( Core.AOS && parent is Mobile )
			{
				Mobile from = (Mobile)parent;

				m_AosSkillBonuses.AddTo( from );

				int strBonus = m_AosAttributes.BonusStr;
				int dexBonus = m_AosAttributes.BonusDex;
				int intBonus = m_AosAttributes.BonusInt;

				if ( strBonus != 0 || dexBonus != 0 || intBonus != 0 )
				{
					string modName = this.Serial.ToString();

					if ( strBonus != 0 )
						from.AddStatMod( new StatMod( StatType.Str, modName + "Str", strBonus, TimeSpan.Zero ) );

					if ( dexBonus != 0 )
						from.AddStatMod( new StatMod( StatType.Dex, modName + "Dex", dexBonus, TimeSpan.Zero ) );

					if ( intBonus != 0 )
						from.AddStatMod( new StatMod( StatType.Int, modName + "Int", intBonus, TimeSpan.Zero ) );
				}

				from.CheckStatTimers();
			}
		}

		public override void OnRemoved( object parent )
		{
			if ( Core.AOS && parent is Mobile )
			{
				Mobile from = (Mobile)parent;

				m_AosSkillBonuses.Remove();

				string modName = this.Serial.ToString();

				from.RemoveStatMod( modName + "Str" );
				from.RemoveStatMod( modName + "Dex" );
				from.RemoveStatMod( modName + "Int" );

				from.CheckStatTimers();
			}
		}

		public BaseJewel( Serial serial ) : base( serial )
		{
		}

		public override void GetProperties( ObjectPropertyList list )
		{
			base.GetProperties( list );
			
			#region JewelCraft System
			if( m_Enchanted && m_Imbued )
				list.Add( 1060847, "\tMagicaly Enchanted & Imbued" ); //Magicaly Enchanted & Imbued
			else if( m_Enchanted )
				list.Add( 1060847, "\tMagicaly Enchanted" ); //Magicaly Enchanted
			else if( m_Imbued )
				list.Add( 1060847, "\tMagicaly Imbued" ); //Magicaly Imbued
			
			if ( m_Crafter != null && JCore.Enabled && JCore.AllowMarker )
				list.Add( 1050043, m_Crafter.Name ); // crafted by ~1_NAME~

			#endregion

			m_AosSkillBonuses.GetProperties( list );

			int prop;

			#region JewelCraft System
			if ( (prop = m_ArtifactRarity) > 0 )
				list.Add( 1061078, prop.ToString() ); // artifact rarity ~1_val~
			#endregion

			if ( (prop = m_AosAttributes.WeaponDamage) != 0 )
				list.Add( 1060401, prop.ToString() ); // damage increase ~1_val~%

			if ( (prop = m_AosAttributes.DefendChance) != 0 )
				list.Add( 1060408, prop.ToString() ); // defense chance increase ~1_val~%

			if ( (prop = m_AosAttributes.BonusDex) != 0 )
				list.Add( 1060409, prop.ToString() ); // dexterity bonus ~1_val~

			if ( (prop = m_AosAttributes.EnhancePotions) != 0 )
				list.Add( 1060411, prop.ToString() ); // enhance potions ~1_val~%

			if ( (prop = m_AosAttributes.CastRecovery) != 0 )
				list.Add( 1060412, prop.ToString() ); // faster cast recovery ~1_val~

			if ( (prop = m_AosAttributes.CastSpeed) != 0 )
				list.Add( 1060413, prop.ToString() ); // faster casting ~1_val~

			if ( (prop = m_AosAttributes.AttackChance) != 0 )
				list.Add( 1060415, prop.ToString() ); // hit chance increase ~1_val~%

			if ( (prop = m_AosAttributes.BonusHits) != 0 )
				list.Add( 1060431, prop.ToString() ); // hit point increase ~1_val~

			if ( (prop = m_AosAttributes.BonusInt) != 0 )
				list.Add( 1060432, prop.ToString() ); // intelligence bonus ~1_val~

			if ( (prop = m_AosAttributes.LowerManaCost) != 0 )
				list.Add( 1060433, prop.ToString() ); // lower mana cost ~1_val~%

			if ( (prop = m_AosAttributes.LowerRegCost) != 0 )
				list.Add( 1060434, prop.ToString() ); // lower reagent cost ~1_val~%

			if ( (prop = m_AosAttributes.Luck) != 0 )
				list.Add( 1060436, prop.ToString() ); // luck ~1_val~

			if ( (prop = m_AosAttributes.BonusMana) != 0 )
				list.Add( 1060439, prop.ToString() ); // mana increase ~1_val~

			if ( (prop = m_AosAttributes.RegenMana) != 0 )
				list.Add( 1060440, prop.ToString() ); // mana regeneration ~1_val~

			if ( (prop = m_AosAttributes.NightSight) != 0 )
				list.Add( 1060441 ); // night sight

			if ( (prop = m_AosAttributes.ReflectPhysical) != 0 )
				list.Add( 1060442, prop.ToString() ); // reflect physical damage ~1_val~%

			if ( (prop = m_AosAttributes.RegenStam) != 0 )
				list.Add( 1060443, prop.ToString() ); // stamina regeneration ~1_val~

			if ( (prop = m_AosAttributes.RegenHits) != 0 )
				list.Add( 1060444, prop.ToString() ); // hit point regeneration ~1_val~

			if ( (prop = m_AosAttributes.SpellChanneling) != 0 )
				list.Add( 1060482 ); // spell channeling

			if ( (prop = m_AosAttributes.SpellDamage) != 0 )
				list.Add( 1060483, prop.ToString() ); // spell damage increase ~1_val~%

			if ( (prop = m_AosAttributes.BonusStam) != 0 )
				list.Add( 1060484, prop.ToString() ); // stamina increase ~1_val~

			if ( (prop = m_AosAttributes.BonusStr) != 0 )
				list.Add( 1060485, prop.ToString() ); // strength bonus ~1_val~

			if ( (prop = m_AosAttributes.WeaponSpeed) != 0 )
				list.Add( 1060486, prop.ToString() ); // swing speed increase ~1_val~%

			base.AddResistanceProperties( list );
		}

		public override void Serialize( GenericWriter writer )
		{
			base.Serialize( writer );

			#region JewelCrafting System
			writer.Write( (int) 4 ); // version
			
			// Version 4
			writer.Write( (int) m_ArtifactRarity ); // Version 4 now Stores the Artifact Rarity...

			// Version 3
			writer.WriteEncodedInt( (int)m_BarType );
			writer.Write( (bool)m_Enchanted );
			writer.Write( (bool)m_Imbued );
			writer.Write( m_OldName );
			writer.Write( (Mobile)m_Crafter );
			#endregion
			
			writer.WriteEncodedInt( (int) m_Resource );
			writer.WriteEncodedInt( (int) m_GemType );

			m_AosAttributes.Serialize( writer );
			m_AosResistances.Serialize( writer );
			m_AosSkillBonuses.Serialize( writer );
		}

		public override void Deserialize( GenericReader reader )
		{
			base.Deserialize( reader );

			int version = reader.ReadInt();

			switch ( version )
			{
				#region JewelCrafting System
				case 4:
				{
					m_ArtifactRarity = (int)reader.ReadInt();
					goto case 3;
				}
				case 3:
				{
					m_BarType = (BarType)reader.ReadEncodedInt();
					m_Enchanted = (bool)reader.ReadBool();
					m_Imbued = (bool)reader.ReadBool();
					m_OldName = reader.ReadString();
					m_Crafter = (Mobile)reader.ReadMobile();
					goto case 2;
				}
				#endregion
				case 2:
				{
					m_Resource = (CraftResource)reader.ReadEncodedInt();
					m_GemType = (GemType)reader.ReadEncodedInt();
					goto case 1;
				}
				case 1:
				{
					m_AosAttributes = new AosAttributes( this, reader );
					m_AosResistances = new AosElementAttributes( this, reader );
					m_AosSkillBonuses = new AosSkillBonuses( this, reader );

					if ( Core.AOS && Parent is Mobile )
						m_AosSkillBonuses.AddTo( (Mobile)Parent );

					int strBonus = m_AosAttributes.BonusStr;
					int dexBonus = m_AosAttributes.BonusDex;
					int intBonus = m_AosAttributes.BonusInt;

					if ( Parent is Mobile && (strBonus != 0 || dexBonus != 0 || intBonus != 0) )
					{
						Mobile m = (Mobile)Parent;

						string modName = Serial.ToString();

						if ( strBonus != 0 )
							m.AddStatMod( new StatMod( StatType.Str, modName + "Str", strBonus, TimeSpan.Zero ) );

						if ( dexBonus != 0 )
							m.AddStatMod( new StatMod( StatType.Dex, modName + "Dex", dexBonus, TimeSpan.Zero ) );

						if ( intBonus != 0 )
							m.AddStatMod( new StatMod( StatType.Int, modName + "Int", intBonus, TimeSpan.Zero ) );
					}

					if ( Parent is Mobile )
						((Mobile)Parent).CheckStatTimers();

					break;
				}
				case 0:
				{
					m_AosAttributes = new AosAttributes( this );
					m_AosResistances = new AosElementAttributes( this );
					m_AosSkillBonuses = new AosSkillBonuses( this );

					break;
				}
			}

			if ( version < 2 )
			{
				m_Resource = CraftResource.Iron;
				m_GemType = GemType.None;
			}
		}
		
		#region ICraftable Members

		public int OnCraft( int quality, bool makersMark, Mobile from, CraftSystem craftSystem, Type typeRes, BaseTool tool, CraftItem craftItem, int resHue )
		{
			if( JCore.Enabled )
				JCore.OnBeforeCraft( from );

			if( makersMark && JCore.Enabled && JCore.AllowMarker )
				this.Crafter = from;
			
			Type resourceType = typeRes;

			if ( resourceType == null )
				resourceType = craftItem.Ressources.GetAt( 0 ).ItemType;

			// If JCore Is Enabled do NOT Obtain the CraftResources
			if( !JCore.Enabled )
				Resource = CraftResources.GetFromType( resourceType );	
			
			#region JewelCrafting System
			if( JCore.Enabled )
			{
				if( resourceType == typeof( CopperBar ) )
					BarType = BarType.Copper;
				else if( resourceType == typeof( SilverBar ) )
					BarType = BarType.Silver;
				else if( resourceType == typeof( GoldBar ) )
					BarType = BarType.Gold;
				else if( resourceType == typeof( PlatinumBar ) )
					BarType = BarType.Platinum;
				else if( resourceType == typeof( VeliumBar ) )
					BarType = BarType.Velium;
				else if( resourceType == typeof( ArycilanBar ) )
					BarType = BarType.Arycilan;
				else if( resourceType == typeof( OrbsydianBar ) )
					BarType = BarType.Orbsydian;
				else if( resourceType == typeof( XulianBar ) )
					BarType = BarType.Xulian;
				else if( resourceType == typeof( ParagonBar ) )
					BarType = BarType.Paragon;
			}
			#endregion
			
			if ( 1 < craftItem.Ressources.Count )
			{
				resourceType = craftItem.Ressources.GetAt( 1 ).ItemType;

				if ( resourceType == typeof( StarSapphire ) )
					GemType = GemType.StarSapphire;
				else if ( resourceType == typeof( Emerald ) )
					GemType = GemType.Emerald;
				else if ( resourceType == typeof( Sapphire ) )
					GemType = GemType.Sapphire;
				else if ( resourceType == typeof( Ruby ) )
					GemType = GemType.Ruby;
				else if ( resourceType == typeof( Citrine ) )
					GemType = GemType.Citrine;
				else if ( resourceType == typeof( Amethyst ) )
					GemType = GemType.Amethyst;
				else if ( resourceType == typeof( Tourmaline ) )
					GemType = GemType.Tourmaline;
				else if ( resourceType == typeof( Amber ) )
					GemType = GemType.Amber;
				else if ( resourceType == typeof( Diamond ) )
					GemType = GemType.Diamond;

				#region JewelCrafting System
				if( JCore.Enabled )
				{
					if ( resourceType == typeof( BlueDiamond ) )
						GemType = GemType.BlueDiamond;
					else if ( resourceType == typeof( FireEmerald ) )
						GemType = GemType.FireEmerald;
					else if ( resourceType == typeof( FireOpal ) )
						GemType = GemType.FireOpal;
					else if ( resourceType == typeof( Jade ) )
						GemType = GemType.Jade;
					else if ( resourceType == typeof( Jasper ) )
						GemType = GemType.Jasper;
					else if ( resourceType == typeof( Onyx ) )
						GemType = GemType.Onyx;
					else if ( resourceType == typeof( Opal ) )
						GemType = GemType.Opal;
					else if ( resourceType == typeof( Pearl ) )
						GemType = GemType.Pearl;
					else if ( resourceType == typeof( StarRose ) )
						GemType = GemType.StarRose;
					else if ( resourceType == typeof( StarRuby ) )
						GemType = GemType.StarRuby;
					else if ( resourceType == typeof( Topaz ) )
						GemType = GemType.Topaz;
					else if ( resourceType == typeof( Turquoise ) )
						GemType = GemType.Turquoise;
				}
				#endregion
			}

			//MOVED to Enchanted Scroll Target..

			if( JCore.Enabled )
				JCore.OnAfterCraft( from );

			return 1;
		}

		#endregion
		
		#region JewelCrafting System
		
		/// <summary>
		/// InvalidateMagicProperties, if the JCore.AOS and JCore.Enabled = false, then it return
		/// If it is IsEnchantable and !m_Enchanted and !m_Imbued is all false, then return
		/// 
		/// Then it gets the JCore.GetNames of the Jewelry
		/// Then it will get the Properties.
		/// </summary>
		public void InvalidateMagicProperties()
		{
			if( !JCore.AOS && !JCore.Enabled )
				return;
			
			if( !IsEnchantable && !m_Enchanted && !m_Imbued )
				return;
			
			JCore.GetNames(this);
			
			if( m_Enchanted && m_Imbued )
			{
				this.GetEnchantImbuedBonus();
			}
			else if( m_Enchanted && !m_Imbued )
			{
				this.GetEnchantBonus();
			}
			else if( !m_Enchanted && m_Imbued )
			{
				this.GetImbuedBonus();
			}					
		}
		
		#region Enchant Bonus
		
		/// <summary>
		/// Gets the Enchanted Bonus properties when Imbued is false
		/// Comes from JCore.GetEnchantBonus
		/// </summary>
		public void GetEnchantBonus()
		{
			JCore.GetEnchantBonus(this);
		}
		
		#endregion
		
		#region Imbued Bonus
		
		/// <summary>
		/// This is from the JCore.GetImbuedBonus, it gets the the Imbued part when Enchanted is false.
		/// </summary>
		public void GetImbuedBonus()
		{
			JCore.GetImbuedBonus( this );
		}
		
		#endregion
		
		#region Enchant & Imbued Bonus
		
		/// <summary>
		/// Gets the Enchant and Imbued Bonuses, this is from the JCore.GetEnchantImbuedBonus
		/// </summary>
		public void GetEnchantImbuedBonus()
		{
			JCore.GetEnchantImbuedBonus( this );
		}
		
		#endregion
		
		/// <summary>
		/// Removes the Magical Properties
		/// If the JCore.Enabled is false, and the JCore.AOS is false, then it will not remove those properties
		/// </summary>
		public void RemoveMagicalProperties()
		{
			if( !JCore.AOS && !JCore.Enabled )
				return;

			if( !this.m_AosAttributes.IsEmpty )
			{
				this.m_AosAttributes.AttackChance = 0;
				this.m_AosAttributes.BonusDex = 0;
				this.m_AosAttributes.BonusHits = 0;
				this.m_AosAttributes.BonusInt = 0;
				this.m_AosAttributes.BonusMana = 0;
				this.m_AosAttributes.BonusStam = 0;
				this.m_AosAttributes.BonusStr = 0;
				this.m_AosAttributes.CastRecovery = 0;
				this.m_AosAttributes.CastSpeed = 0;
				this.m_AosAttributes.DefendChance = 0;
				this.m_AosAttributes.EnhancePotions = 0;
				this.m_AosAttributes.LowerManaCost = 0;
				this.m_AosAttributes.LowerRegCost = 0;
				this.m_AosAttributes.Luck = 0;
				this.m_AosAttributes.NightSight = 0;
				this.m_AosAttributes.ReflectPhysical = 0;
				this.m_AosAttributes.RegenHits = 0;
				this.m_AosAttributes.RegenMana = 0;
				this.m_AosAttributes.RegenStam = 0;
				this.m_AosAttributes.SpellChanneling = 0;
				this.m_AosAttributes.SpellDamage = 0;
				this.m_AosAttributes.WeaponDamage = 0;
				this.m_AosAttributes.WeaponSpeed = 0;
			}

			if( !this.m_AosResistances.IsEmpty )
			{
				this.m_AosResistances.Cold = 0;
				this.m_AosResistances.Energy = 0;
				this.m_AosResistances.Fire = 0;
				this.m_AosResistances.Physical = 0;
				this.m_AosResistances.Poison = 0;
			}
			
			if( !this.m_AosSkillBonuses.IsEmpty )
				this.m_AosSkillBonuses.Remove();
			
			this.Name = this.OldName;
		}
		
		/// <summary>
		/// Gets the Bonus of the BarType
		/// </summary>
		/// <returns>returns the BarType value of 0 and just returns the full BarType Value</returns>
		public int GetBonus()
		{
			return GetBonus( 0 );
		}
		
		/// <summary>
		/// Gets the Bonus of the BarType
		/// </summary>
		/// <param name="bonus">Default: return 0</param>
		/// <returns>returns the BarType value of the bonus and BarType</returns>
		public int GetBonus( int bonus )
		{			
			switch( m_BarType )
			{
				case BarType.None: default: return 0;
				case BarType.Copper: return JCore.CopperBonus+bonus;
				case BarType.Silver: return JCore.SilverBonus+bonus;
				case BarType.Gold: return JCore.GoldBonus+bonus;
				case BarType.Platinum: return JCore.PlatinumBonus+bonus;
				case BarType.Velium: return JCore.VeliumBonus+bonus;
				case BarType.Arycilan: return JCore.ArycilanBonus+bonus;
				case BarType.Orbsydian: return JCore.OrbsydianBonus+bonus;
				case BarType.Xulian: return JCore.XulianBonus+bonus;
				case BarType.Paragon: return JCore.ParagonBonus+bonus;
			}
		}
		
		/// <summary>
		/// Gets the Luck of the BarType, where it returns 0
		/// </summary>
		/// <returns>returns the Luck to be 0, where it just gives the BarType Luck</returns>
		public int GetLuck()
		{
			return GetLuck( 0 );
		}
		
		/// <summary>
		/// Gets the Luck of the BarType
		/// </summary>
		/// <param name="bonus">Default: returns 0</param>
		/// <returns>Gets the BarType Luck.</returns>
		public int GetLuck( int bonus )
		{			
			switch( m_BarType )
			{
				case BarType.None: default: return 0;
				case BarType.Copper: return JCore.CopperLuck+bonus;
				case BarType.Silver: return JCore.SilverLuck+bonus;
				case BarType.Gold: return JCore.GoldLuck+bonus;
				case BarType.Platinum: return JCore.PlatinumLuck+bonus;
				case BarType.Velium: return JCore.VeliumLuck+bonus;
				case BarType.Arycilan: return JCore.ArycilanLuck+bonus;
				case BarType.Orbsydian: return JCore.OrbsydianLuck+bonus;
				case BarType.Xulian: return JCore.XulianLuck+bonus;
				case BarType.Paragon: return JCore.ParagonLuck+bonus;
			}
		}
		
		/// <summary>
		/// Sets the Bonus to 0, where it gets just the Resistnace
		/// </summary>
		/// <returns>returns 0, where it just returns the BarType Resistance</returns>
		public int GetResistances()
		{
			return GetResistances( 0 );
		}

        /// <summary>
        /// Gets the Resistance Bonus of a BarType is used in the Jewelry Elemental Resistance
        /// </summary>
        /// <param name="bonus">Default: return 0</param>
        /// <returns>Returns the BarType Resistance</returns>  		
		public int GetResistances( int bonus )
		{
			switch( m_BarType )
			{
				case BarType.None: default: return 0;
				case BarType.Copper: return JCore.CopperResistance+bonus;
				case BarType.Silver: return JCore.SilverResistance+bonus;
				case BarType.Gold: return JCore.GoldResistance+bonus;
				case BarType.Platinum: return JCore.PlatinumResistance+bonus;
				case BarType.Velium: return JCore.VeliumResistance+bonus;
				case BarType.Arycilan: return JCore.ArycilanResistance+bonus;
				case BarType.Orbsydian: return JCore.OrbsydianResistance+bonus;
				case BarType.Xulian: return JCore.XulianResistance+bonus;
				case BarType.Paragon: return JCore.ParagonResistance+bonus;
			}
		}
		
		/// <summary>
		/// This sets the Percentage bonus to 0
		/// </summary>
		/// <returns>bonus returns 0 and returns just the bartype percentage</returns>
		public int GetPercent()
		{
			return GetPercent( 0 );
		}
		
		/// <summary>
		/// Get the Percentage of each BarType for the AOSAttributes
		/// </summary>
		/// <param name="bonus">Default: return 0</param>
		/// <returns>Returns the BarType Get Percentage</returns>
		public int GetPercent( int bonus )
		{
			switch( m_BarType )
			{
				case BarType.None: default: return 0;
				case BarType.Copper: return JCore.CopperPercent+bonus;
				case BarType.Silver: return JCore.SilverPercent+bonus;
				case BarType.Gold: return JCore.GoldPercent+bonus;
				case BarType.Platinum: return JCore.PlatinumPercent+bonus;
				case BarType.Velium: return JCore.VeliumPercent+bonus;
				case BarType.Arycilan: return JCore.ArycilanPercent+bonus;
				case BarType.Orbsydian: return JCore.OrbsydianPercent+bonus;
				case BarType.Xulian: return JCore.XulianPercent+bonus;
				case BarType.Paragon: return JCore.ParagonPercent+bonus;
			}
		}
		
		/// <summary>
		/// Gets the Bonus of the Bars
		/// </summary>
		/// <returns>This will set the bonus to 0, and just return the attcual stats.</returns>
		public int GetStats()
		{
			return GetStats( 0 );
		}
		
        /// <summary>
        /// Gets the Bonus of the Bars
        /// </summary>
        /// <param name="bonus">Default: return 0</param>
        /// <returns>Returns the BarType Get Stats.</returns>
		public int GetStats( int bonus )
		{
			switch( m_BarType )
			{
				case BarType.None: default: return 0;
				case BarType.Copper: return JCore.CopperStats+bonus;
				case BarType.Silver: return JCore.SilverStats+bonus;
				case BarType.Gold: return JCore.GoldStats+bonus;
				case BarType.Platinum: return JCore.PlatinumStats+bonus;
				case BarType.Velium: return JCore.VeliumStats+bonus;
				case BarType.Arycilan: return JCore.ArycilanStats+bonus;
				case BarType.Orbsydian: return JCore.OrbsydianStats+bonus;
				case BarType.Xulian: return JCore.XulianStats+bonus;
				case BarType.Paragon: return JCore.ParagonStats+bonus;
			}
		}		
		
		/// <summary>
		/// Default: CanEnchant = true.
		/// 
		/// If CanEnchant = true, then it will check, otherwise it will automatically be false.
		/// </summary>
		public virtual bool CanEnchant{ get{ return true; } }

		/// <summary>
		/// If it is Resource is None, and the AosAttributes etc. are NOT Emtpy
		/// Then it is Magical, which means you CANNOT Enchant or Imbued!
		/// If it is Enchanted or Imbued it remains Is not Magical (When it really is)
		/// If it is Enchanted and Imbued, it becomes Magical
		/// If CanEnchant = true, then it will check, otherwise it will automatically be false.
		/// </summary>
		[CommandProperty( AccessLevel.GameMaster )]
		public bool IsEnchantable
		{
			get
			{	
				if( !CanEnchant )
					return false;

				if( this.m_Resource == CraftResource.None &&
				    this.m_AosAttributes.IsEmpty && 
				    this.m_AosResistances.IsEmpty && 
				    this.m_AosSkillBonuses.IsEmpty  )
					return true;			
				
					return false;
			}
		}
		
		#endregion
	}
}
